--let's create separate scene and hold everything there
--then it will be easier to reuse it if you want to use SceneManager class
scene = gideros.class(Sprite)

--on scene initialization
function scene:init()
	--create world instance
	self.world = b2.World.new(0, 10, true)
	
	local screenW = application:getContentWidth()
	local screenH = application:getContentHeight()
	--create bounding walls outside the scene
	self:wall(0,screenH/2,10,screenH)
	self:wall(screenW/2,0,screenW,10)
	self:wall(screenW,screenH/2,10,screenH)
	self:wall(screenW/2,screenH,screenW,10)
	
	--create ball
	self.ball = self:ball(200, 200)
	self:addChild(self.ball)
	
	self.world:createCircle(self.ball, {type = "dynamic", draggable = true})
	
	--add collision event listener
	self.world:addEventListener(Event.BEGIN_CONTACT, self.onBeginContact, self)
	
	--run world
	self:addEventListener(Event.ENTER_FRAME, self.onEnterFrame, self)
end

--define begin collision event handler function
function scene:onBeginContact(e)

	--getting contact bodies
	local fixtureA = e.fixtureA
	local fixtureB = e.fixtureB
	local bodyA = fixtureA:getBody()
	local bodyB = fixtureB:getBody()
	
	--check if first colliding body is a ball
	if bodyB.type and bodyB.type == "ball" then
		--switch animation using our created function
		bodyB.object:switchAnimation()
	end
end

-- for creating objects using shape
-- as example - bounding walls
function scene:wall(x, y, width, height)
	local wall = Shape.new()
	--define wall shape
	wall:beginPath()
	
	--we make use (0;0) as center of shape,
	--thus we have half of width and half of height in each direction
	wall:moveTo(-width/2,-height/2)
	wall:lineTo(width/2, -height/2)
	wall:lineTo(width/2, height/2)
	wall:lineTo(-width/2, height/2)
	wall:closePath()
	wall:endPath()
	wall:setPosition(x,y)
	
	--create box2d physical object
	local body = self.world:createBody{type = b2.STATIC_BODY}
	body:setPosition(wall:getX(), wall:getY())
	body:setAngle(wall:getRotation() * math.pi/180)
	local poly = b2.PolygonShape.new()
	poly:setAsBox(wall:getWidth()/2, wall:getHeight()/2)
	local fixture = body:createFixture{shape = poly, density = 1.0, 
	friction = 0.1, restitution = 0.8}
	wall.body = body
	wall.body.type = "wall"
	
	--add to scene
	self:addChild(wall)
	
	--return created object
	return wall
end

-- for creating objects using movieclip
function scene:ball(x, y)
	--of course it all can be done through loops or other abstractions
	--but for the sake of demonstration, I'll doo all images one by one
	
	--get eyses animation images
	local eyes1 = Bitmap.new(Texture.new("animation/eyes1.png", true))
	local eyes2 = Bitmap.new(Texture.new("animation/eyes2.png", true))
	local eyes3 = Bitmap.new(Texture.new("animation/eyes3.png", true))
	local eyes4 = Bitmap.new(Texture.new("animation/eyes4.png", true))
	local eyes5 = Bitmap.new(Texture.new("animation/eyes5.png", true))
	local eyes6 = Bitmap.new(Texture.new("animation/eyes6.png", true))
	--get smile animation images
	local smile0 = Bitmap.new(Texture.new("animation/smile0.png", true))
	local smile1 = Bitmap.new(Texture.new("animation/smile1.png", true))
	local smile2 = Bitmap.new(Texture.new("animation/smile2.png", true))
	local smile3 = Bitmap.new(Texture.new("animation/smile3.png", true))
	local smile4 = Bitmap.new(Texture.new("animation/smile4.png", true))
	
	--setting anchor points
	eyes1:setAnchorPoint(0.5, 0.5)
	eyes2:setAnchorPoint(0.5, 0.5)
	eyes3:setAnchorPoint(0.5, 0.5)
	eyes4:setAnchorPoint(0.5, 0.5)
	eyes5:setAnchorPoint(0.5, 0.5)
	eyes6:setAnchorPoint(0.5, 0.5)
	smile0:setAnchorPoint(0.5, 0.5)
	smile1:setAnchorPoint(0.5, 0.5)
	smile2:setAnchorPoint(0.5, 0.5)
	smile3:setAnchorPoint(0.5, 0.5)
	smile4:setAnchorPoint(0.5, 0.5)
	
	--creating movieclip
	local ball = MovieClip.new{
		{1, 10, eyes1}, 
		{11, 20, eyes2}, 
		{21, 30, eyes3}, 
		{31, 40, eyes4}, 
		{41, 50, eyes5}, 
		{51, 60, eyes6}, 
		{61, 70, smile0}, 
		{71, 80, smile1}, 
		{81, 90, smile2}, 
		{91, 100, smile3}, 
		{101, 110, smile4}
	}
	
	--set looping of first animation
	ball:setGotoAction(60, 1)
	
	--set looping of second animation
	ball:setGotoAction(110, 61)
	
	--goto first image and play
	ball:gotoAndPlay(1)
	
	--mark that first animation is playing
	ball.firstAnimation = true
	
	--function to switch animations
	function ball:switchAnimation()
		if self.firstAnimation then
			--play second animation
			--which starts from frame 61
			self:gotoAndPlay(61)
		else
			--play first animation
			self:gotoAndPlay(1)
		end
		
		--change mark of current animation
		self.firstAnimation = not self.firstAnimation
	end
	
	--example of stopping animation on specific frame
	--in this case frame number 81
	function ball:stopAnimation()
		self:gotoAndStop(81)
	end
	
	--set position
	ball:setPosition(x,y)
	
	--return created object
	return ball
end

--running the world
function scene:onEnterFrame() 
	self.world:update()
end

--add created scene to stage or sceneManager
local mainScene = scene.new()
stage:addChild(mainScene)